#include "CentralWidget.h"
#include <QtGui>
#include <fstream>
using namespace std;

CentralWidget::CentralWidget(QWidget* parent) : QWidget(parent)
{
	unitSymbol = "";
	unitAfterNumber = true;

	stackedGraphWidget = new StackedGraphWidget(&data, &unitSymbol, &unitAfterNumber, &unitSizeNames, this);
	stackedGraphWidgetCompare = new StackedGraphWidget(&data, &unitSymbol, &unitAfterNumber, &unitSizeNames, this);
	stackedGraphWidgetCompare->hide();

	QVBoxLayout* layout = new QVBoxLayout(this);
	layout->addWidget(stackedGraphWidget);
	layout->addWidget(stackedGraphWidgetCompare);
}

CentralWidget::~CentralWidget()
{

}

void CentralWidget::loadData(const QString& filePath)
{
	data.clear();
	unitSizeNames.clear();

	ifstream textFile;
	textFile.open(filePath.toStdString().c_str(), ifstream::in);

	if (textFile.good()) {
		const char unitSettingsStart = '[';
		const char unitSettingsSeparator = '|';
		const char unitSettingsEnd = ']';
		const char dataStart = '{';
		const char dataEnd = '}';
		const char dataDelimiter = ',';
		const char labelStart = '"';
		const char labelEnd = '"';
		const char dataPartStart = '(';
		const char dataPartEnd = ')';
		string tempText;

		getline(textFile,tempText,unitSettingsStart);
		getline(textFile,tempText,labelStart);
		getline(textFile,tempText,labelEnd);
		unitSymbol = QString::fromStdString(tempText);

		getline(textFile,tempText,unitSettingsSeparator);
		cleanString(tempText,dataDelimiter);
		cleanString(tempText,'\r');
		cleanString(tempText,'\n');
		cleanString(tempText,' ');
		cleanString(tempText,'	');
		if (tempText == "false" || tempText == "FALSE") {
			unitAfterNumber = false;
		} else {
			unitAfterNumber = true;
		}

		getline(textFile,tempText,unitSettingsEnd);
		if (!tempText.empty()) {
			while (tempText.find(labelStart) < tempText.size()) {
				unitSizeNames.push_back(QString::fromStdString(tempText.substr(tempText.find(labelStart)+1, tempText.find(labelEnd,tempText.find(labelStart)+1)-1-tempText.find(labelStart))));
				tempText.erase(0,tempText.find(labelEnd,tempText.find(labelStart)+1)+1);
			}
		}

		getline(textFile,tempText,dataStart);
		while (textFile.good()) {
			getline(textFile,tempText,labelStart);
			getline(textFile,tempText,labelEnd);

			DataItem item;
			item.label = QString::fromStdString(tempText);

			getline(textFile,tempText,dataStart);
			getline(textFile,tempText,dataEnd);

			cleanString(tempText,'\r');
			cleanString(tempText,'\n');
			cleanString(tempText,' ');
			cleanString(tempText,'	');
			if (!tempText.empty()) {
				while (tempText.find(dataPartStart) < tempText.size()) {
					item.data.push_back(stringToDataItemPart(tempText.substr(tempText.find(dataPartStart)+1, tempText.find(dataPartEnd,tempText.find(dataPartStart)+1)-1-tempText.find(dataPartStart)), dataDelimiter));
					tempText.erase(0,tempText.find(dataPartEnd,tempText.find(dataPartStart)+1)+1);
				}
			}

			data.push_back(item);

			getline(textFile,tempText,dataStart);
		}

		for (int i=0; i<data.size(); i++) {
			if (data[i].data.size() < 2) {
				data.remove(i);
				i--;
			}
		}
		for (int i=0; i<data.size(); i++) {
			qSort(data[i].data);
		}
		qSort(data);
		for (int i=0; i<data.size(); i++) {
			linearInterpolate(data[i].data);
		}
	}

	textFile.close();

	stackedGraphWidget->updateStackedGraph();
	stackedGraphWidgetCompare->updateStackedGraph();
}

void CentralWidget::cleanString(std::string& text, const char charToDelete) {
	while (text.find(charToDelete) < text.size()) {
		text.erase(text.find(charToDelete),1);
	}
}

DataItemPart CentralWidget::stringToDataItemPart(std::string text, const char dataDelimiter) {
	DataItemPart itemPart;

	if (text.find(dataDelimiter) < text.size()) {
		itemPart.year = atoi(text.substr(0,text.find(dataDelimiter)).c_str());
		text.erase(0,text.find(dataDelimiter)+1);
	}
	if (text.find(dataDelimiter) < text.size()) {
		itemPart.month = atoi(text.substr(0,text.find(dataDelimiter)).c_str());
		text.erase(0,text.find(dataDelimiter)+1);
		if (itemPart.month < 1) {
			itemPart.month = 1;
		} else if (itemPart.month > 12) {
			itemPart.month = 12;
		}
	}
	itemPart.quantity = atof(text.c_str());
	if (itemPart.quantity < 0.0) {
		itemPart.quantity = 0.0;
	}

	return itemPart;
}

void CentralWidget::linearInterpolate(QVector<DataItemPart>& items)
{
	for (int i=0; i<items.size()-1; i++) {
		while (i+1 < items.size() && items[i] == items[i+1]) {
			items.remove(i+1);
		}
		if (i+1 < items.size()) {
			int first = items[i].year * 12 + items[i].month;
			double firstQuantity = items[i].quantity;
			int second = items[i+1].year * 12 + items[i+1].month;
			double secondQuantity = items[i+1].quantity;
			int diff = second - first;
			for (int j=1; j<diff; j++) {
				DataItemPart item;

				int year = items[i].year;
				int month = items[i].month + j;
				while (month > 12) {
					year++;
					month -= 12;
				}

				double position = double(j) / double(diff);
				double quantity = firstQuantity + position * (secondQuantity - firstQuantity);

				item.year = year;
				item.month = month;
				item.quantity = quantity;
				items.insert(i+j, item);
			}
		}
	}
}

void CentralWidget::setGLViewport(bool openGL)
{
	stackedGraphWidget->getStackedGraph()->getView()->setGLViewport(openGL);
	stackedGraphWidgetCompare->getStackedGraph()->getView()->setGLViewport(openGL);
}

void CentralWidget::setAutoGLViewport(bool autoGL)
{
	stackedGraphWidget->getStackedGraph()->getView()->setAutoGLViewport(autoGL);
	stackedGraphWidgetCompare->getStackedGraph()->getView()->setAutoGLViewport(autoGL);
}

void CentralWidget::setDetailedInformation(bool detailInfo)
{
	stackedGraphWidget->getStackedGraph()->setDetailedInformation(detailInfo);
	stackedGraphWidgetCompare->getStackedGraph()->setDetailedInformation(detailInfo);
}

void CentralWidget::toggleCompareGraph(bool compareGraph)
{
	if (compareGraph) {
		*stackedGraphWidgetCompare = *stackedGraphWidget;
		stackedGraphWidgetCompare->show();
	} else {
		stackedGraphWidgetCompare->hide();
	}
}
