#include "ColorScheme1D.h"

ColorScheme1D::ColorScheme1D(void)
{
	// create a grayscale colorscheme
	this->color1 = QColor("lightGray");
	this->color2 = QColor("black");
}

ColorScheme1D::ColorScheme1D(const QColor& color1)
{
	// create a hue based colorscheme
	this->color1 = color1;
	this->color2 = QColor();
}

ColorScheme1D::ColorScheme1D(const QColor& color1, const QColor& color2)
{
	// create a 2-color interpolation based colorscheme
	this->color1 = color1;
	this->color2 = color2;
}

// SETTERS

void ColorScheme1D::setColor_1( const QColor& color1 )
{
	this->color1 = color1;
}

void ColorScheme1D::setColor_2( const QColor& color2 )
{
	this->color2 = color2;
}

void ColorScheme1D::setColors( const QColor& color1, const QColor& color2 )
{
	this->color1 = color1;
	this->color2 = color2;
}

// GETTERS

QColor ColorScheme1D::getColor(double percentage){
	// distinguish colorschemes
	if ( !(this->color2.isValid()) ){
		// Interpolate colors with a "constant" HUE
		return QColor("gray");
	}
	if ( (this->color1.isValid()) && (this->color2.isValid()) ){
		// interpolate colors in HSV color model
		QColor outputColor;
		int newHue;

		//if (this->color1.hsvHue() > this->color2.hsvHue()){
		// watch out! not solved yet
		//newHue =  ((int)( (this->color1.hsvHue() * (percentage)) + (this->color2.hsvHue() * (1-percentage)) )) % 360;
		//} else {
		newHue =  (int)( (this->color1.hsvHue() * (percentage)) + (this->color2.hsvHue() * (1-percentage)) );
		//}
		int newSat =  (int)( (this->color1.saturation() * percentage) + (this->color2.saturation() * (1-percentage)) );
		int newValue =(int)( (this->color1.value() * percentage) + (this->color2.value() * (1-percentage)) );
		int newAlpha =(int)( (this->color1.alpha() * percentage) + (this->color2.alpha() * (1-percentage)) );
		outputColor.setHsv(newHue, newSat, newValue, newAlpha);
		return outputColor;
	}
	// in every other case :-)
	return QColor("red");
}

QColor ColorScheme1D::getContrastColor(double percentage)
{
	QColor actualColor = this->getColor(percentage);
	QColor newColor;
	int newValue;

	if (actualColor.value()<140) {
		newValue = 255;
	} else {
		newValue = 0;
	}
	int newAlpha = 255; // always opaque

	newColor.setHsv(0, 0, newValue, newAlpha);
	return newColor;
}

// destructor
ColorScheme1D::~ColorScheme1D(void)
{
}
