#pragma once

#include <QColor>

/**
* This class is a representation of a 1D ColorScheme.
* 1D means that you can get colors from an one dimensional "color space"
* which	 is created by the user by specifying a 2 color color ramp.
*/
class ColorScheme1D
{
public:

	ColorScheme1D(void); ///< creates a grayscale colorscheme
	ColorScheme1D(const QColor& color1); ///< creates a hue based "one color" colorscheme
	ColorScheme1D(const QColor& color1, const QColor& color2); ///< creates an HSV interpolated color scheme

	virtual ~ColorScheme1D(void);

	/**
	* \brief returns the requested color according to the created colorscheme.
	* \param percentage floating point value between 0.0 and 1.0.
	*			A value close to 0.0 means the color is close to color1 and color2 has no influence.
	*			A value close to 1.0 means the color is close to color2 and color1 has no influence.
	*/
	QColor getColor(double percentage);

	/**
	* \brief returns a grayscale color that has a high contrast against the color you would get with calling getColor method.
	* \param percentage floating point value between 0.0 and 1.0.
	*			A value close to 0.0 means the color is close to color1 and color2 has no influence.
	*			A value close to 1.0 means the color is close to color2 and color1 has no influence.
	*/
	QColor getContrastColor(double percentage);

	void setColor_1(const QColor& color1); ///< sets the first colorschemecolor to the color given
	void setColor_2(const QColor& color2); ///< sets the second colorschemecolor to the color given
	void setColors(const QColor& color1, const QColor& color2); ///< sets the first and second colorschemecolor to the two colors given

private:
	QColor color1; ///< first color of colorscheme
	QColor color2; ///< second color of colorscheme
};
