#include "MainWindow.h"
#include <QGLFormat>
#include <QtGui>

MainWindow::MainWindow()
{
	title = "Stacked Graph";
	// set up the menuBar
	// FILE
	QMenu* menuBarFile = this->menuBar()->addMenu(tr("File"));

	QAction* menuBarFileLoadData = new QAction(menuBarFile);
	menuBarFileLoadData->setText(tr("Load Data"));
	connect(menuBarFileLoadData, SIGNAL(triggered()), this, SLOT(setOpenFileName()));

	QAction* menuBarFileQuit = new QAction(menuBarFile);
	menuBarFileQuit->setText(tr("Quit"));
	connect(menuBarFileQuit, SIGNAL(triggered()), this, SIGNAL(quitApp()));

	menuBarFile->addAction(menuBarFileLoadData);
	menuBarFile->addSeparator();
	menuBarFile->addAction(menuBarFileQuit);

	QMenu* menuBarView = this->menuBar()->addMenu(tr("View"));

	QAction* menuBarViewAutoOpenGL = new QAction(menuBarView);
	menuBarViewAutoOpenGL->setText(tr("Auto OpenGL"));
	menuBarViewAutoOpenGL->setCheckable(true);

	QAction* menuBarViewOpenGL = new QAction(menuBarView);
	menuBarViewOpenGL->setText(tr("OpenGL"));
	menuBarViewOpenGL->setCheckable(true);

	QAction* menuBarViewDetailedInformation = new QAction(menuBarView);
	menuBarViewDetailedInformation->setText(tr("Detailed Information"));
	menuBarViewDetailedInformation->setCheckable(true);

	QAction* menuBarViewCompareGraph = new QAction(menuBarView);
	menuBarViewCompareGraph->setText(tr("Compare Graph"));
	menuBarViewCompareGraph->setCheckable(true);

	menuBarView->addAction(menuBarViewAutoOpenGL);
	menuBarView->addAction(menuBarViewOpenGL);
	menuBarView->addSeparator();
	menuBarView->addAction(menuBarViewDetailedInformation);
	menuBarView->addSeparator();
	menuBarView->addAction(menuBarViewCompareGraph);

	// ABOUT
	QMenu* menuBarInfo = this->menuBar()->addMenu(tr("Info"));
	QAction* showAbout = new QAction(menuBarInfo);
	showAbout->setText(tr("About"));
	connect(showAbout, SIGNAL(triggered()), this, SLOT(showAboutBox()));
	menuBarInfo->addAction(showAbout);

	centralWidget = new CentralWidget(this);
	connect(menuBarViewOpenGL, SIGNAL(toggled(bool)), centralWidget, SLOT(setGLViewport(bool)));
	connect(menuBarViewAutoOpenGL, SIGNAL(toggled(bool)), menuBarViewOpenGL, SLOT(setDisabled(bool)));
	connect(menuBarViewAutoOpenGL, SIGNAL(toggled(bool)), menuBarViewOpenGL, SLOT(setChecked(bool)));
	connect(menuBarViewAutoOpenGL, SIGNAL(toggled(bool)), centralWidget, SLOT(setAutoGLViewport(bool)));
	connect(menuBarViewDetailedInformation, SIGNAL(toggled(bool)), centralWidget, SLOT(setDetailedInformation(bool)));
	connect(menuBarViewCompareGraph, SIGNAL(toggled(bool)), centralWidget, SLOT(toggleCompareGraph(bool)));
	if (!QGLFormat::hasOpenGL() || !centralWidget->hasGLAA()) {
		menuBarViewAutoOpenGL->setEnabled(false);
		menuBarViewOpenGL->setEnabled(false);
	} else {
		menuBarViewAutoOpenGL->setChecked(true);
	}

	this->setCentralWidget(centralWidget);
	this->setDockOptions(QMainWindow::AnimatedDocks | QMainWindow::ForceTabbedDocks | QMainWindow::VerticalTabs);
	this->setWindowTitle(title);
}

MainWindow::~MainWindow()
{

}

void MainWindow::setOpenFileName()
{
	QString filePath = QFileDialog::getOpenFileName(this, tr("Load File"), "", tr("Data Files (*.data);;All Files (*)"));

	if (!filePath.isEmpty()) {
		centralWidget->loadData(filePath);
		filePath = " (" + filePath.section('/',-1).section('.',0,0) + ")";
		this->setWindowTitle(title+filePath);
	}
}

void MainWindow::showAboutBox()
{
	QMessageBox::about(this, "About", "\nwritten by:\n\nGeorg Schleicher	\nWolfgang Ludwig	\n");
}
