#ifndef STACKEDGRAPHVIEW_H
#define STACKEDGRAPHVIEW_H

#include <QGraphicsView>
#include <QGraphicsScene>

/**
* This class is a view for the stacked graph.
*/
class StackedGraphView : public QGraphicsView
{
	Q_OBJECT

public:
	StackedGraphView(QGraphicsScene* scene, QWidget* parent); ///< creates a view for the stacked graph
	~StackedGraphView();

	bool hasGLAA() {return hasGLSampleBuffer;} ///< returns if GL anti-aliasing is available

signals:
	void toolTipChanged(QGraphicsItem* item, QPoint pos); ///< this signal is send when a tool tip should be shown

public slots:
	void showWholeGraph(); ///< resets the view to show the whole graph
	/**
	* \brief set the viewport of this view.
	* \param openGL if true, the viewport is switched to openGL redering (if possible). if false, the viewport is set to software rendering.
	*/
	void setGLViewport(bool openGL);
	/**
	* \brief set the auto GL feature of this view. The auto GL mode has just effects, if openGL is supported, and GL anti-aliasing is possible.
	* \param autoGL if true, activates auto GL (if possible). if false, deactivates auto GL.
	*/
	void setAutoGLViewport(bool autoGL) {autoGLViewport = autoGL; checkGLViewport();}

protected:
	void wheelEvent(QWheelEvent* event); ///< handles the mouse wheel events
	void mouseReleaseEvent(QMouseEvent* event); ///< handles the mouse button release events

private slots:
	void resetView() {this->fitInView(this->sceneRect(),Qt::KeepAspectRatio);} ///< resets the view to show the whole graph
	void resetPixelScale(); ///< resets the pixelScale

private:
	void checkGLViewport(); ///< this is the autoGL check. it decides when to switch to openGL rendering, and when to switch to software rendering.

	bool hasGLSampleBuffer;
	bool hasGLViewport;
	bool autoGLViewport;
	double pixelScale;
};

#endif
