#include "StackedGraphWidget.h"

StackedGraphWidget::StackedGraphWidget(const QVector<DataItem>* dataPtr, const QString* unitSymbolPtr, const bool* unitAfterNumberPtr, const QVector<QString>* unitSizeNamesPtr, QWidget* parent) : QWidget(parent)
{
	QGroupBox* menu = new QGroupBox("Menu", this);

	QGroupBox* menuExport = new QGroupBox("Export", menu);

	QPushButton* saveImageButton = new QPushButton("Export Graph to Image", menuExport);
	connect(saveImageButton, SIGNAL(clicked()), this, SLOT(saveImage()));

	imageSizeBox = new QComboBox(menuExport);
	imageSizeBox->addItem("0");
	imageSizeBox->addItem("1");
	imageSizeBox->addItem("2");
	imageSizeBox->addItem("3");
	imageSizeBox->addItem("4");

	imageAlphaBox = new QCheckBox(menuExport);

	QFormLayout* menuExportLayout = new QFormLayout(menuExport);
	menuExportLayout->addRow("Imagesize:", imageSizeBox);
	menuExportLayout->addRow("Transparency:", imageAlphaBox);
	menuExportLayout->addRow(saveImageButton);

	QGroupBox* menuGraph = new QGroupBox("Graph Visualization", menu);

	baselineAlgoBox = new QComboBox(menuGraph);
	baselineAlgoBox->addItem("x-axis");
	baselineAlgoBox->addItem("symmetric");
	baselineAlgoBox->addItem("baseline");

	sortAlgoBox = new QComboBox(menuGraph);
	sortAlgoBox->addItem("onset-time");
	sortAlgoBox->addItem("inside-out");
	sortAlgoBox->addItem("weighted");

	colorAlgoBox = new QComboBox(menuGraph);
	colorAlgoBox->addItem("first onset");
	colorAlgoBox->addItem("last offset");
	colorAlgoBox->addItem("max quant");
	colorAlgoBox->addItem("avrg quant");
	colorAlgoBox->addItem("sum quant");

	colorpicker1 = new ColorPushButton(menuGraph);
	connect(colorpicker1, SIGNAL(clicked()), this, SLOT(setColor1()));

	colorpicker2 = new ColorPushButton(menuGraph);
	connect(colorpicker2, SIGNAL(clicked()), this, SLOT(setColor2()));

	colorSchemeBox = new QComboBox(menuGraph);
	colorSchemeBox->addItem("custom");
	QPixmap pixmap(16,16);
	QPainter painter(&pixmap);
	
	pixmap.fill(QColor(48,48,48));
	painter.fillRect(0, 8, 16, 8, QColor(150,150,150));
	colorSchemeBox->addItem(pixmap,"grayscale");

	pixmap.fill(QColor(246,82,82));
	painter.fillRect(0, 8, 16, 8, QColor(255,246,75));
	colorSchemeBox->addItem(pixmap,"sunset");
	
	pixmap.fill(QColor(153,211,255));
	painter.fillRect(0, 8, 16, 8, QColor(220,255,255));
	colorSchemeBox->addItem(pixmap,"winter");

	pixmap.fill(QColor(93,170,16));
	painter.fillRect(0, 8, 16, 8, QColor(255,246,75));
	colorSchemeBox->addItem(pixmap,"lime");

	pixmap.fill(QColor(96,122,170));
	painter.fillRect(0, 8, 16, 8, QColor(255,255,127));
	colorSchemeBox->addItem(pixmap,"yellow2blue");

	pixmap.fill(QColor(83,32,33));
	painter.fillRect(0, 8, 16, 8, QColor(217,47,50));
	colorSchemeBox->addItem(pixmap,"red blood");

	pixmap.fill(QColor(16,129,69));
	painter.fillRect(0, 8, 16, 8, QColor(178,255,169));
	colorSchemeBox->addItem(pixmap,"green");

	pixmap.fill(QColor(172,131,20));
	painter.fillRect(0, 8, 16, 8, QColor(251,251,172));
	colorSchemeBox->addItem(pixmap,"sand");

	QFormLayout* menuGraphLayout = new QFormLayout(menuGraph);
	menuGraphLayout->addRow("Baseline:", baselineAlgoBox);
	menuGraphLayout->addRow("Sorting:", sortAlgoBox);
	menuGraphLayout->addRow("Coloring:", colorAlgoBox);
	menuGraphLayout->addRow("Max Color:", colorpicker1);
	menuGraphLayout->addRow("Min Color:", colorpicker2);
	menuGraphLayout->addRow("Colorscheme:", colorSchemeBox);

	QVBoxLayout* menuLayout = new QVBoxLayout(menu);
	menuLayout->addWidget(menuGraph);
	menuLayout->addWidget(menuExport);
	menuLayout->addStretch(1);

	xRangeSlider = new QSlider(Qt::Vertical, this);
	xRangeSlider->setRange(160,640);
	xRangeSlider->setSingleStep(100);
	xRangeSlider->setPageStep(1000);
	xRangeSlider->setInvertedAppearance(true);
	xRangeSlider->setInvertedControls(true);

	xRangeSliderTimer = new QTimer(this);
	connect(xRangeSliderTimer, SIGNAL(timeout()), this, SLOT(xRangeSliderAction()));

	stackedGraph = new StackedGraph(dataPtr, unitSymbolPtr, unitAfterNumberPtr, unitSizeNamesPtr, this);
	stackedGraph->setRenderGraph(false);
	connect(baselineAlgoBox, SIGNAL(currentIndexChanged(int)), stackedGraph, SLOT(setBaselineAlgo(int)));
	connect(baselineAlgoBox, SIGNAL(currentIndexChanged(int)), this, SLOT(makeImageSizeBoxText()));
	connect(sortAlgoBox, SIGNAL(currentIndexChanged(int)), stackedGraph, SLOT(setSortAlgo(int)));
	connect(sortAlgoBox, SIGNAL(currentIndexChanged(int)), this, SLOT(makeImageSizeBoxText()));
	connect(colorAlgoBox, SIGNAL(currentIndexChanged(int)), stackedGraph, SLOT(setColorAlgo(int)));
	connect(colorpicker1, SIGNAL(colorChanged(const QColor&)), stackedGraph, SLOT(setMaxColor(const QColor&)));
	connect(colorpicker2, SIGNAL(colorChanged(const QColor&)), stackedGraph, SLOT(setMinColor(const QColor&)));
	connect(stackedGraph, SIGNAL(maxColorTextChanged(const QString&,const QColor&)), colorpicker1, SLOT(setColorText(const QString&,const QColor&)));
	connect(stackedGraph, SIGNAL(minColorTextChanged(const QString&,const QColor&)), colorpicker2, SLOT(setColorText(const QString&,const QColor&)));
	connect(colorSchemeBox, SIGNAL(currentIndexChanged(int)), this, SLOT(setColorScheme()));
	connect(xRangeSlider, SIGNAL(sliderPressed()), this, SLOT(startXRangeSlider()));
	connect(xRangeSlider, SIGNAL(sliderReleased()), this, SLOT(stopXRangeSlider()));
	connect(xRangeSlider, SIGNAL(valueChanged(int)), this, SLOT(xRangeSliderActionTest()));
	imageSizeBox->setCurrentIndex(2);
	baselineAlgoBox->setCurrentIndex(2);
	sortAlgoBox->setCurrentIndex(2);
	colorAlgoBox->setCurrentIndex(4);
	colorpicker1->setColor(Qt::gray);
	colorpicker2->setColor(Qt::black);
	colorSchemeBox->setCurrentIndex(1);
	xRangeSlider->setValue(320);

	QHBoxLayout* layout = new QHBoxLayout(this);
	layout->addWidget(menu);
	layout->addWidget(xRangeSlider);
	layout->addWidget(stackedGraph->getView());

	stackedGraph->setRenderGraph(true);
	updateStackedGraph();
}

StackedGraphWidget::~StackedGraphWidget()
{
}

StackedGraphWidget& StackedGraphWidget::operator=(const StackedGraphWidget& other)
{
	if (this != &other) {
		stackedGraph->setRenderGraph(false);
		baselineAlgoBox->setCurrentIndex(other.baselineAlgoBox->currentIndex());
		sortAlgoBox->setCurrentIndex(other.sortAlgoBox->currentIndex());
		colorAlgoBox->setCurrentIndex(other.colorAlgoBox->currentIndex());
		xRangeSlider->setRange(other.xRangeSlider->minimum(), other.xRangeSlider->maximum());
		xRangeSlider->setValue(other.xRangeSlider->value());
		colorpicker1->setColor(other.colorpicker1->getColor());
		colorpicker2->setColor(other.colorpicker2->getColor());
		colorSchemeBox->setCurrentIndex(other.colorSchemeBox->currentIndex());
		stackedGraph->setRenderGraph(true);
		stackedGraph->makeStackedGraph();
		stackedGraph->getView()->showWholeGraph();
		makeImageSizeBoxText();
	}
	return *this;
}

void StackedGraphWidget::updateStackedGraph()
{
	stackedGraph->setRenderGraph(false);
	int xRange = stackedGraph->getLength() * 320;
	xRangeSlider->setRange(xRange * 0.5, xRange * 2);
	xRangeSlider->setValue(xRange);
	stackedGraph->setRenderGraph(true);
	stackedGraph->makeStackedGraph();
	stackedGraph->getView()->showWholeGraph();
	makeImageSizeBoxText();
}

void StackedGraphWidget::makeImageSizeBoxText()
{
	imageSizeBox->setItemText(0,QString::number(xRangeSlider->value()/4)+"x"+QString::number(stackedGraph->getYRange()/4));
	imageSizeBox->setItemText(1,QString::number(xRangeSlider->value()/2)+"x"+QString::number(stackedGraph->getYRange()/2));
	imageSizeBox->setItemText(2,QString::number(xRangeSlider->value())+"x"+QString::number(stackedGraph->getYRange()));
	imageSizeBox->setItemText(3,QString::number(xRangeSlider->value()*2)+"x"+QString::number(stackedGraph->getYRange()*2));
	imageSizeBox->setItemText(4,QString::number(xRangeSlider->value()*4)+"x"+QString::number(stackedGraph->getYRange()*4));
}

void StackedGraphWidget::saveImage()
{
	QString filePath = QFileDialog::getSaveFileName(this, tr("save Image"), "", tr("PNG (*.png);;All Files (*)"));
	if (!filePath.isEmpty()) {
		/*if (!filePath.endsWith(".png",Qt::CaseInsensitive)) {
			filePath += ".png";
		}*/
		int imageWidth = 16;
		int imageHeight = 10;
		if (imageSizeBox->currentIndex() == 0) {
			imageWidth = xRangeSlider->value() / 4;
			imageHeight = stackedGraph->getYRange() / 4;
		} else if (imageSizeBox->currentIndex() == 1) {
			imageWidth = xRangeSlider->value() / 2;
			imageHeight = stackedGraph->getYRange() / 2;
		} else if (imageSizeBox->currentIndex() == 2) {
			imageWidth = xRangeSlider->value();
			imageHeight = stackedGraph->getYRange();
		} else if (imageSizeBox->currentIndex() == 3) {
			imageWidth = xRangeSlider->value() * 2;
			imageHeight = stackedGraph->getYRange() * 2;
		} else if (imageSizeBox->currentIndex() == 4) {
			imageWidth = xRangeSlider->value() * 4;
			imageHeight = stackedGraph->getYRange() * 4;
		}
		QImage::Format format = QImage::Format_RGB32;
		if (imageAlphaBox->isChecked()) {
			format = QImage::Format_ARGB32;
		}
		QImage image(imageWidth,imageHeight,format);
		image.fill(0);
		image.invertPixels();
		QPainter painter(&image);
		painter.setRenderHint(QPainter::Antialiasing);
		stackedGraph->render(&painter);
		image.save(filePath,"PNG");
	}
}

void StackedGraphWidget::setColor1()
{
	colorpicker1->setColor(QColorDialog::getColor(colorpicker1->getColor(),this));
}

void StackedGraphWidget::setColor2()
{
	colorpicker2->setColor(QColorDialog::getColor(colorpicker2->getColor(),this));
}

void StackedGraphWidget::setColorScheme()
{
	if (colorSchemeBox->currentIndex() > 0) {
		bool update = true;
		if (!stackedGraph->getRenderGraph()) {update = false;}
		stackedGraph->setRenderGraph(false);
		disconnect(colorpicker1, SIGNAL(colorChanged(const QColor&)), this, SLOT(resetColorSchemeBox()));
		disconnect(colorpicker2, SIGNAL(colorChanged(const QColor&)), this, SLOT(resetColorSchemeBox()));
		if (colorSchemeBox->currentIndex() == 1) {
			// colorscheme "grayscale"
			colorpicker1->setColor(QColor(48,48,48));
			colorpicker2->setColor(QColor(150,150,150));
		} else if (colorSchemeBox->currentIndex() == 2) {
			// colorscheme "sunset"
			colorpicker1->setColor(QColor(246,82,82));
			colorpicker2->setColor(QColor(255,246,75));
		} else if (colorSchemeBox->currentIndex() == 3) {
			// colorscheme "winter"
			colorpicker1->setColor(QColor(153,211,255));
			colorpicker2->setColor(QColor(220,255,255));
		} else if (colorSchemeBox->currentIndex() == 4) {
			// colorscheme "lime"
			colorpicker1->setColor(QColor(93,170,16));
			colorpicker2->setColor(QColor(255,246,75));
		} else if (colorSchemeBox->currentIndex() == 5) {
			// colorscheme "yellow2blue"
			colorpicker1->setColor(QColor(96,122,170));
			colorpicker2->setColor(QColor(255,255,127));
		} else if (colorSchemeBox->currentIndex() == 6) {
			// colorscheme "red blood"
			colorpicker1->setColor(QColor(83,32,33));
			colorpicker2->setColor(QColor(217,47,50));
		} else if (colorSchemeBox->currentIndex() == 7) {
			// colorscheme "green"
			colorpicker1->setColor(QColor(16,129,69));
			colorpicker2->setColor(QColor(178,255,169));
		} else if (colorSchemeBox->currentIndex() == 8) {
			// colorscheme "sand"
			colorpicker1->setColor(QColor(172,131,20));
			colorpicker2->setColor(QColor(251,251,172));
		}
		connect(colorpicker1, SIGNAL(colorChanged(const QColor&)), this, SLOT(resetColorSchemeBox()));
		connect(colorpicker2, SIGNAL(colorChanged(const QColor&)), this, SLOT(resetColorSchemeBox()));
		if (update) {stackedGraph->setRenderGraph(true);}
		stackedGraph->makeStackedGraph();
	}
}

void StackedGraphWidget::xRangeSliderActionTest()
{
	if (!xRangeSlider->isSliderDown()) {
		xRangeSliderAction();
	}
}

void StackedGraphWidget::xRangeSliderAction()
{
	stackedGraph->setXRange(xRangeSlider->value());
	if (stackedGraph->getRenderGraph()) {
		stackedGraph->getView()->showWholeGraph();
	}
	makeImageSizeBoxText();
}
