#ifndef STACKEDGRAPHWIDGET_H
#define STACKEDGRAPHWIDGET_H

#include <QWidget>
#include <QtGui>
#include <QTimer>
#include "ColorPushButton.h"
#include "StackedGraph.h"
#include "DataItem.h"

/**
* This class is a widget which contains a stacked graph and a menu for this graph.
*/
class StackedGraphWidget : public QWidget
{
	Q_OBJECT

public:
	/**
	* \brief creates a widget which contains a stacked graph and a menu for this graph.
	* \param dataPtr the data which should be used for the stacked graph.
	* \param unitSymbolPtr the unit symbol for the data.
	* \param unitAfterNumberPtr if the unit symbol should be shown before or after the number.
	* \param unitSizeNamesPtr the size names for each 1000 step.
	* \param parent the parent widget.
	*/
	StackedGraphWidget(const QVector<DataItem>* dataPtr, const QString* unitSymbolPtr, const bool* unitAfterNumberPtr, const QVector<QString>* unitSizeNamesPtr, QWidget* parent);
	~StackedGraphWidget();

	StackedGraphWidget& operator=(const StackedGraphWidget& other); ///< copies a stackedGraphWidget

	StackedGraph* getStackedGraph() {return stackedGraph;} ///< returns the pointer to the stackedGraph

public slots:
	void updateStackedGraph(); ///< updates the stackedGraph

private slots:
	void makeImageSizeBoxText(); ///< updates the the text in the imageSizeBox
	void saveImage(); ///< starts a QFileDialog and saves the image at the selected location
	void setColor1(); ///< sets the first color of the graphs color scheme
	void setColor2(); ///< sets the second color of the graphs color scheme
	void setColorScheme(); ///< sets one of the predefined color schemes
	void resetColorSchemeBox() {colorSchemeBox->setCurrentIndex(0);} ///< sets the color colorSchemeBox to custom
	void startXRangeSlider() {xRangeSliderTimer->start(50);} ///< starts the graph rendering every 50ms
	void stopXRangeSlider() {xRangeSliderTimer->stop(); xRangeSliderAction();} ///< stops the graph rendering
	void xRangeSliderActionTest(); ///< only calls xRangeSliderAction() if the slider button is not pressed
	void xRangeSliderAction(); ///< the action which should be executed when the xRangeSlider is used

private:
	StackedGraph* stackedGraph;
	ColorPushButton* colorpicker1;
	ColorPushButton* colorpicker2;
	QCheckBox* imageAlphaBox;
	QComboBox* imageSizeBox;
	QComboBox* baselineAlgoBox;
	QComboBox* sortAlgoBox;
	QComboBox* colorAlgoBox;
	QComboBox* colorSchemeBox;
	QSlider* xRangeSlider;
	QTimer* xRangeSliderTimer;
};

#endif
